// { dg-do run { target c++11 } }

int main() {
  int i;
  const char* s;
  [i, s] () -> void { i; s; } ();

  //[] () -> void { i; } (); // { dg-error: "`i' is not in scope" }
  //[1] () -> void {} (); // { dg-error: "expected identifier" }

  return 0;
}

