// PR c++/13269
// { dg-options "-fshow-column" }

class Foo { int foo() return 0; } };

// { dg-error "30:expected identifier before numeric constant" "identifier" { target *-*-* } 4 }

// { dg-error "23:named return values are no longer supported" "named return" { target *-*-* } 4 }

// the column number info of this error output is still wrong because the error
// message has been generated by cp_parser_error() which does not
// necessarily allow accurate column number display. At some point, we will
// need make cp_parser_error() report more accurate column numbers.
// { dg-error "30:expected '\{' at end of input" "brace" { target *-*-* } 4 }

// { dg-error "33:expected ';' after class definition" "semicolon" {target *-*-* } 4 }

// { dg-error "35:expected declaration before '\}' token" "declaration" {target *-*-* } 4 }

