unsigned _ov_64_seek_lap (_Bool x1, _Bool x2, _Bool x3)
{
  unsigned ltmp_3978_7__PHI_TEMPORARY;
  signed ltmp_4011_7;

  if (!x1 || !x2)
    while (1) ;

  if (x3)
    ltmp_3978_7__PHI_TEMPORARY = 0xffffff7e;
  else
    ltmp_3978_7__PHI_TEMPORARY = 1;

  ltmp_4011_7 = -1;
  return ltmp_4011_7 >> ltmp_3978_7__PHI_TEMPORARY;
}

